"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
* Author: Chrioyles
* Date: 06.09.2018
**/
var EZStore = /*#__PURE__*/function () {
  /**
  * PUBLIC FUNCTIONS
  **/

  /**
  * Create the store while filling it with data
  **/
  function EZStore(_ref) {
    var data = _ref.data,
        events = _ref.events;

    _classCallCheck(this, EZStore);

    if (!(_typeof(data) === 'object')) {
      console.warn('Error in module EZStore. Value passed to constructor has to be' + 'typeof Object.');
    }

    this._defaultStore = JSON.stringify(data);
    this._listeners = [];
    this._data = _objectSpread({}, data);
    this._events = _objectSpread({}, events);
    this._subscribeIndex = 0;
  }

  _createClass(EZStore, [{
    key: "keys",

    /**
     * Returns all registered keys of the store as a an object with key values equal to the keys.
     */
    value: function keys() {
      var _this = this;

      return Object.keys(this._data).filter(function (key) {
        return _this._data.hasOwnProperty(key);
      }).reduce(function (keys, key) {
        return _objectSpread({}, keys, _defineProperty({}, key, key));
      }, {});
    }
    /**
     * Returns all registered event names of the store as a an object with key values equal to the keys.
     */

  }, {
    key: "eventNames",
    value: function eventNames() {
      var _this2 = this;

      return Object.keys(this._events).filter(function (key) {
        return _this2._events.hasOwnProperty(key);
      }).reduce(function (keys, key) {
        return _objectSpread({}, keys, _defineProperty({}, key, key));
      }, {});
    }
    /**
     * Event dispatcher
     */

  }, {
    key: "dispatch",
    value: function dispatch(_ref2) {
      var eventName = _ref2.eventName,
          payload = _ref2.payload;

      if (this._events.hasOwnProperty(eventName)) {
        this._events[eventName](payload);
      } else {
        console.warn("Event:", eventName, "is not a registered event in EZStore");
      }
    }
    /**
     * Restores the default store object
     */

  }, {
    key: "restoreDefault",
    value: function restoreDefault() {
      try {
        this._data = _objectSpread({}, JSON.parse(this._defaultStore));
      } catch (e) {
        console.warn("Could not reset the store to default: ", e);
      }
    }
    /**
     * Clears all listeners.
     */

  }, {
    key: "clearListeners",
    value: function clearListeners() {
      this._listeners = [];
    }
    /**
    * Set a value to a key in the store. Key must match a key in the _data Object.
    **/

  }, {
    key: "set",
    value: function set(key, value) {
      if (!(typeof key === 'string') || this._getDeepObjectByString(this._data, key) === undefined) {
        console.warn('Error in module EZStore. ' + 'The key passed to EZStore.set() is not found in the store');
      }

      if (value === this._getDeepObjectByString(this._data, key)) return;

      this._setDeepObjectByString(this._data, key, value);

      this._dispatchChange(this._listeningOn(key));
    }
  }, {
    key: "get",

    /**
    * Get a value of a key in the store. Key must match a key in the _data Object.
    **/
    value: function get(key) {
      if (!(typeof key === 'string') || this._getDeepObjectByString(this._data, key) === undefined) {
        console.warn('Error in module EZStore. ' + 'The key passed to EZStore.get() is not found in the store');
        return;
      }

      ;
      return Object.assign({}, {
        data: this._getDeepObjectByString(this._data, key)
      }).data;
    }
  }, {
    key: "subscribe",

    /**
    * Subscribe to a key in the store. Triggers a change when the value of the key
    * changes
    **/
    value: function subscribe(key, cb) {
      var once = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (!cb) {
        console.warn('Error in module EZStore. ' + 'Subscribe function did not receive a callback function');
        return -1;
      }

      if (!key) {
        console.warn('Error in module EZStore. Subscribe function did not receive a key');
        return -1;
      }

      var listener = {};
      listener[key] = cb;
      listener.index = this._subscribeIndex++;
      listener.once = once;

      this._listeners.push(listener);

      return listener.index;
    }
  }, {
    key: "notifyListeners",

    /**
     * Trigger a dispatch of the store key.
     **/
    value: function notifyListeners(key) {
      this._dispatchChange(this._listeningOn(key));
    }
    /**
    * Deletes a listener recognized by its subscriptionId/index.
    **/

  }, {
    key: "unsubscribe",
    value: function unsubscribe(index) {
      if (index === undefined || index === null || !(typeof index === 'number')) {
        console.warn('Error in module EZStore. Unsubscribe function expects a parameter of type number');
        return -1;
      }

      this._listeners = this._listeners.filter(function (listener) {
        return listener.index !== index;
      });
    }
    /**
    * PRIVATE FUNCTIONS
    **/

  }, {
    key: "_getDeepObjectByString",
    value: function _getDeepObjectByString(obj, string) {
      var i = 0;
      var stringArr = string.split('.');

      for (i = 0; i < stringArr.length - 1; i++) {
        obj = obj[stringArr[i]];
      }

      return obj[stringArr[i]];
    }
  }, {
    key: "_setDeepObjectByString",
    value: function _setDeepObjectByString(obj, string, value) {
      var i = 0;
      var stringArr = string.split('.');

      for (i = 0; i < stringArr.length - 1; i++) {
        obj = obj[stringArr[i]];
      }

      obj[stringArr[i]] = value;
    }
    /**
    * Filters out all subscriber function to the key specified as a parameter.
    **/

  }, {
    key: "_listeningOn",
    value: function _listeningOn(key) {
      var _this3 = this;

      if (!(typeof key === 'string')) return;
      return this._keyChain(key).map(function (partialKey) {
        return _this3._listeners.filter(function (listener) {
          return listener.hasOwnProperty(partialKey);
        });
      }).reduce(function (map, item) {
        return map.concat(item);
      }, []);
    }
  }, {
    key: "_parentKey",

    /**
     * Get the parent key of a key
     */
    value: function _parentKey(key) {
      return key.split(".").reduce(function (parentKey, subKey, idx, arr) {
        return parentKey + (idx !== arr.length - 1 ? ".".concat(subKey) : "");
      }, "").replace(/^\./, "");
    }
    /**
     * Creates a chain of keys like: 'key1.key2.key3' => ['key1.key2.key3', 'key1.key2', 'key1']
     */

  }, {
    key: "_keyChain",
    value: function _keyChain(key) {
      var arr = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

      var parentKey = this._parentKey(key);

      var newArray = [].concat(_toConsumableArray(arr), [key]);
      return parentKey ? this._keyChain(parentKey, newArray) : newArray;
    }
    /**
    * Calls all functions of the listeners specified in the parameters.
    * Clears all listeners that should execute only once.
    **/

  }, {
    key: "_dispatchChange",
    value: function _dispatchChange(listeners) {
      var _this4 = this;

      var that = this;
      listeners.forEach(function (listener) {
        var keyName = Object.keys(listener)[0];

        if (keyName) {
          listener[keyName](that._getDeepObjectByString(that._data, keyName));
          if (listener.once) _this4.unsubscribe(listener.index);
        }
      });
    }
  }]);

  return EZStore;
}();

var _default = EZStore;
exports["default"] = _default;